Introduction
===

AppDynamics’ Node.js agent allows you to monitor your Node.js applications in production so you get real-time visibility of server health, application performance, and resource metrics.

The causes of performance problems can be varied and complex to identify while the results usually boil down to a slow or unavailable web site. The AppDynamics Node.js agent reveals the internals of your application and infrastructure through Node.js process-level visibility and proactive monitoring using policies, enabling detailed analysis, fast troubleshooting, performance and capacity optimization. The agent’s process snapshots let you examine your code execution at a multi-process level so you can find the root cause of slow performance in minutes.

Supported Environments
--
[Read our support matrix](http://docs.appdynamics.com/display/PRO14S/Supported+Environments+and+Versions+for+Node.js) for details on the supported Node.js versions, operating systems, and backends. The supported environments are constantly being updated.

Requirements
--
In order to use the AppDynamics Node.js agent, you will need the following:

-   An AppDynamics account with a running controller. [Sign up for an account here](https://portal.appdynamics.com/account/). 
-   You will receive an email with your controller *host* and *port*. 
- To download and install AppDynamics on your own, you may [download the controller from here](http://download.appdynamics.com/browse/zone/3/) and then [follow these instructions on how to install](http://docs.appdynamics.com/display/PRO39/Install+the+Controller).


Installing the Node.js agent
--
You may install the latest version by typing the following command from the root directory of the Node.js application that you are instrumenting: 

```sh
npm install appdynamics
```

If you know which specific version of the Node.js agent you want to install, you may define it:

```sh
npm install appdynamics@<version>
```

This command assumes your app is built on the most recent Node.js version. If that is not the case, [use the instructions here](http://docs.appdynamics.com/display/PRO39/Install+the+App+Agent+for+Node.js).

Then paste the following in your application  as the very first line of your application source code, before any other require statement:  

```sh
require("appdynamics").profile({
  controllerHostName: '<controller host name>',
  // If SSL, be sure to set controllerSslEnabled to true
  controllerPort: <controller port number>, 
  controllerSslEnabled: false, 
  // Required for a controller running in multi-tenant mode
  accountName: '<AppDynamics account name>', 
  // Required for a controller running in multi-tenant mode
  accountAccessKey: '<AppDynamics account key>', 
  applicationName: '<app_name>',
  tierName: '<tier_name>', 
  nodeName: '<node_name>', 
  debug: false
 });
```

[See here for further installation details](http://docs.appdynamics.com/display/PRO39/Install+the+App+Agent+for+Node.js).

For More Information
--
[Click here general information about monitoring and troubleshooting using AppDynamics](http://docs.appdynamics.com/display/PRO39/AppDynamics+Pro+Documentation)
.

